-- get localised difficulty names
local nameNormal, _, _, _, _, _, _ = GetDifficultyInfo(1)
local nameHeroic, _, _, _, _, _, _ = GetDifficultyInfo(2)
local nameMythic, _, _, _, _, _, _ = GetDifficultyInfo(23)
local nameNormalRaid, _, _, _, _, _, _ = GetDifficultyInfo(14)
local nameHeroicRaid, _, _, _, _, _, _ = GetDifficultyInfo(15)
local nameMythicRaid, _, _, _, _, _, _ = GetDifficultyInfo(16)

-- options
local filterCleared = false
local filterPartialCleared = false
local filterNoDpsSlot = false
local filterNoHealSlot = false
local filterNoTankSlot = false
local applicationText = nil

-- these will hold our lockout infos
local dungeonLockouts = {}
local raidLockouts = {}

-- this will hold our result item infos
local resultInfos = {}

local function LoadLockouts()
	-- clear old lockout infos
	dungeonLockouts = {}
	raidLockouts = {}
	
	-- scan for lockouts
	for i = 1, GetNumSavedInstances() do
		local name, id, _, difficulty, locked, extended, _, isRaid, maxPlayers, difficultyName, numEncounters, numDefeated  = GetSavedInstanceInfo(i)
		if locked or extended then
			local clear = true
			local encounters = nil
			
			-- if not all encounters are done, store done encounters for later comparison
			if numDefeated ~= numEncounters then
				clear = false
				local bossName, isKilled
				encounters = {}
				for k = 1, numEncounters do
					bossName, _, isKilled, _ = GetSavedInstanceEncounterInfo(i, k)
					encounters[bossName] = isKilled
				end
			end
			
			-- build info data structure
			local info = {
				isClear = clear,
				tEncounters = encounters,
				isRaid = isRaid
			}

			if isRaid then
				if difficulty == 14 then -- normal
					raidLockouts[name ..' ('..nameNormalRaid..')'] = info
				elseif difficulty == 16 then -- mythic
					raidLockouts[name ..' ('..nameMythicRaid..')'] = info
				elseif difficulty == 15 then -- heroic
					raidLockouts[name ..' ('..nameHeroicRaid..')'] = info
				end
			else
				if difficulty == 23 then -- mythic
					dungeonLockouts[name ..' ('..nameMythic..')'] = info
				elseif difficulty == 2 then -- heroic
					dungeonLockouts[name ..' ('..nameHeroic..')'] = info
				end
			end
		end
	end
end

local function CheckResult(self)
	local resultID = self;
	local id, activityID, name, comment, voiceChat, iLvl, honorLevel, age, numBNetFriends, numCharFriends, numGuildMates, isDelisted, leaderName, numMembers = C_LFGList.GetSearchResultInfo(resultID)
	local color = 0
	local info = nil
	local hide = false
	if not isDelisted then
		color = -3
		local fullName, shortName, categoryID, groupID, iLevel, filters, minLevel, maxPlayers, displayType = C_LFGList.GetActivityInfo(activityID)
		info = dungeonLockouts[fullName]
		if info == nil then
			info = raidLockouts[fullName]			
		end
		if info then
			if info.isClear then
				color = -1
				if filterCleared then
					hide = true
				end
			else
				color = -2
				if filterPartialCleared then
					hide = true
				end
				local completedEncounters = C_LFGList.GetSearchResultEncounterInfo(resultID)
				if completedEncounters then
					local n = #completedEncounters
					local encounters = info.tEncounters
					if  encounters and n > 0 then
						for i=1, n do
							if not encounters[completedEncounters[i]] then
								color = -1
								break
							end
						end
					end
				end
			end
		end
		if not hide and displayType == LE_LFG_LIST_DISPLAY_TYPE_ROLE_ENUMERATE then
			local memberCounts = C_LFGList.GetSearchResultMemberCounts(resultID)
			if (filterNoDpsSlot and memberCounts.DAMAGER == 3)
			or (filterNoHealSlot and memberCounts.HEALER == 1)
			or (filterNoTankSlot and memberCounts.TANK == 1)
			then
				hide = true
			end
		end
	end
	resultInfos[resultID] = {
		color = color,
		info = info,
		hide = hide
	}
	return not hide
end

local function hook_LFGListSearchPanel_UpdateResultList(self)
	if self.searching or self.searchFailed then
		return
	end
	local results = self.results
	local nr = #results
	resultInfos = {}
	if nr == 0 then
		return
	end
	local out = {}
	for i=1, nr do
		local result = results[i]
		if result then
			if CheckResult(result) then
				out[#out+1] = result
			end
		end
	end
	self.results = out
end

local function hook_LFGListSearchEntry_Update(self)
	local color = nil
	local resultID = self.resultID
	local data = resultInfos[resultID]
	if data then
		color = data.color
		if data.hide then
			return
		end
		local id, activityID, name, comment, voiceChat, iLvl, honorLevel, age, numBNetFriends, numCharFriends, numGuildMates, isDelisted = C_LFGList.GetSearchResultInfo(resultID);
		if color and not isDelisted then
			if color == -1 then
				self.ActivityName:SetTextColor(0.77, 0.12, 0.23);
			elseif color == -2 then
				self.ActivityName:SetTextColor(1, 0.49, 0.04);
			elseif color == -3 then
				self.ActivityName:SetTextColor(0.67, 0.83, 0.45);
			else
				self.ActivityName:SetTextColor(0.67, 0.83, 0.45);
			end
		end
	end
end

local function hook_FilterButton_OnClick(self)
	local entry = UIDropDownMenu_CreateInfo()
	entry.text = " "
	entry.disabled = false
	entry.isTitle = true
	UIDropDownMenu_AddButton(entry)
	entry = UIDropDownMenu_CreateInfo()
	entry.text = HIDE..":"
	entry.disabled = false
	entry.isTitle = true
	UIDropDownMenu_AddButton(entry)
	entry = UIDropDownMenu_CreateInfo()
	entry.text = "locked and cleared"
	entry.checked = filterCleared
	entry.disabled = false
	entry.isNotRadio = true
	entry.keepShownOnClick = true
	entry.func = function(self,_,_,checked) filterCleared = checked end
	UIDropDownMenu_AddButton(entry)
	entry = UIDropDownMenu_CreateInfo()
	entry.text = "locked and partial cleared"
	entry.checked = filterPartialCleared
	entry.disabled = false
	entry.isNotRadio = true
	entry.keepShownOnClick = true
	entry.func = function(self,_,_,checked) filterPartialCleared = checked end
	UIDropDownMenu_AddButton(entry)
	entry = UIDropDownMenu_CreateInfo()
	entry.text = "no Tank slot"
	entry.checked = filterTankDpsSlot
	entry.disabled = false
	entry.isNotRadio = true
	entry.keepShownOnClick = true
	entry.func = function(self,_,_,checked) filterTankDpsSlot = checked end
	UIDropDownMenu_AddButton(entry)
	entry = UIDropDownMenu_CreateInfo()
	entry.text = "no Healer slot"
	entry.checked = filterHealDpsSlot
	entry.disabled = false
	entry.isNotRadio = true
	entry.keepShownOnClick = true
	entry.func = function(self,_,_,checked) filterHealDpsSlot = checked end
	UIDropDownMenu_AddButton(entry)
	entry = UIDropDownMenu_CreateInfo()
	entry.text = "no Dps slot"
	entry.checked = filterNoDpsSlot
	entry.disabled = false
	entry.isNotRadio = true
	entry.keepShownOnClick = true
	entry.func = function(self,_,_,checked) filterNoDpsSlot = checked end
	UIDropDownMenu_AddButton(entry)
end

local function hook_PVEFrame_ShowFrame(sidePanelName, selection)
	LoadLockouts()
end

local function hook_LFGListSearchEntry_OnEnter(self)
	if not GameTooltip:IsShown() then return end
	local resultID = self.resultID;
	local data = resultInfos[resultID]
	if data and not data.hide then
		local info = data.info
		if info then
			if not info.isClear and info.tEncounters then
				local enounters = info.tEncounters
				GameTooltip:AddLine(" ")
				GameTooltip:AddLine(BOSS_ALIVE..":")
				for key,value in pairs(enounters) do
					if not value then
						GameTooltip:AddLine(key, GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b, true)
					end
				end
			end
		end
		local id, activityID, name, comment, voiceChat, iLvl, honorLevel, age, numBNetFriends, numCharFriends, numGuildMates, isDelisted, leaderName, numMembers = C_LFGList.GetSearchResultInfo(resultID)
		if not isDelisted then
			GameTooltip:AddLine(" ")
			GameTooltip:AddLine(CLASS_ROLES..":")
			--if numMembers > 5 then
				local roleData = {}
				for i=1, numMembers do
					local role, class, classLocalized = C_LFGList.GetSearchResultMemberInfo(resultID, i);
					if not roleData[role] then
						roleData[role] = {}
					end
					if not roleData[role][class] then
						roleData[role][class] = { 1, classLocalized}
					else
						roleData[role][class][1] = roleData[role][class][1] + 1
					end
				end
				for role, data in pairs(roleData) do
					local text = _G[role]..": "
					for class, classData in pairs(data) do
						local classColor = RAID_CLASS_COLORS[class] or NORMAL_FONT_COLOR;
						local count = ""
						if classData[1] > 1 then
							count = classData[1].."x"
						end
						text = text.."|c"..classColor.colorStr..count..classData[2].."|r "
						
					end
					GameTooltip:AddLine(text);
				end
			--else
			--	for i=1, numMembers do
			--		local role, class, classLocalized = C_LFGList.GetSearchResultMemberInfo(resultID, i);
			--		local classColor = RAID_CLASS_COLORS[class] or NORMAL_FONT_COLOR;
			--		GameTooltip:AddLine(string.format(LFG_LIST_TOOLTIP_CLASS_ROLE, classLocalized, _G[role]), classColor.r, classColor.g, classColor.b);
			--	end
			--end
		end
		GameTooltip:Show()
	end
end

local initSearchPanel = false
local function hook_LFGListFrame_SetActivePanel(self, panel)
	if not initSearchPanel and panel == self.SearchPanel then
		local buttons = self.SearchPanel.ScrollFrame.buttons
		for i=1, #buttons do
			buttons[i]:HookScript("OnEnter", hook_LFGListSearchEntry_OnEnter)
		end
		self.SearchPanel.FilterButton:HookScript("OnClick", hook_FilterButton_OnClick)
		initSearchPanel = true
	end
end

local function hook_LFGListApplicationDialog_Show(self)
	local txt = self.Description.EditBox:GetText()
	if not txt or txt == "" and applicationText then
		self.Description.EditBox:SetText(applicationText);
	end
	self.SignUpButton:HookScript("OnClick", function(self) applicationText = self:GetParent().Description.EditBox:GetText() end)
end

local frame = CreateFrame("FRAME");
frame:RegisterEvent("ADDON_LOADED");
frame:RegisterEvent("PLAYER_LOGOUT");

function frame:OnEvent(event, arg1)
 if event == "ADDON_LOADED" and arg1 == "LFDLockouts" then
	if LFDLockoutsCharData == nil then
		LFDLockoutsCharData = {
			filterCleared = filterCleared,
			filterPartialCleared = filterPartialCleared,
			filterNoDpsSlot = filterNoDpsSlot,
			filterNoHealSlot = filterNoHealSlot,
			filterNoTankSlot = filterNoTankSlot,
			applicationText = applicationText,
		}
	else
		filterCleared = LFDLockoutsCharData.filterCleared and true or false
		filterNoDpsSlot = LFDLockoutsCharData.filterNoDpsSlot and true or false
		applicationText = LFDLockoutsCharData.applicationText
  end
	
	hooksecurefunc("LFGListApplicationDialog_Show", hook_LFGListApplicationDialog_Show)
	hooksecurefunc("LFGListSearchPanel_UpdateResultList", hook_LFGListSearchPanel_UpdateResultList)
	hooksecurefunc("LFGListSearchEntry_Update", hook_LFGListSearchEntry_Update)
	hooksecurefunc("PVEFrame_ShowFrame", hook_PVEFrame_ShowFrame)
	hooksecurefunc("LFGListFrame_SetActivePanel", hook_LFGListFrame_SetActivePanel)
	hooksecurefunc("LFGListSearchEntry_OnEnter", hook_LFGListSearchEntry_OnEnter)
 elseif event == "PLAYER_LOGOUT" then
		LFDLockoutsCharData = {
			filterCleared = filterCleared,
			filterPartialCleared = filterPartialCleared,
			filterNoDpsSlot = filterNoDpsSlot,
			filterNoHealSlot = filterNoHealSlot,
			filterNoTankSlot = filterNoTankSlot,
			applicationText = applicationText,
		}
 end
end
frame:SetScript("OnEvent", frame.OnEvent);